@@ -95,7 +95,10 @@ CFSocketContext gSockContext;
 /** current mode */
 SInt32 gMode = -1;
 #endif
+
 CFStringRef gIMName = NULL;
+Boolean gEnableModeTips = true;
+Boolean gCandVertical = true;
 
 static MenuRef gPencilMenu;
 
@@ -139,6 +142,7 @@ MUIMInitialize(ComponentInstance inComponentInstance,
   Handle iconData = NULL;
   Handle menuIconSuite;
   Str255 menuText;
+  Boolean dummy;
 
   result = noErr;
   refNum = -1;
@@ -243,6 +247,18 @@ MUIMInitialize(ComponentInstance inComponentInstance,
   DEBUG_PRINT("MUIMInitialize()\n");
 #endif
 
+  if (CFPreferencesGetAppBooleanValue(CFSTR(kPrefHelperIM),
+                                      CFSTR(kAppID), &dummy))
+    gCandVertical = true;
+  else
+    gCandVertical = false;
+ 
+  if (CFPreferencesGetAppBooleanValue(CFSTR(kPrefModeTips),
+                                      CFSTR(kAppID), &dummy))
+    gEnableModeTips = true;
+  else
+    gEnableModeTips = false;
+  
   {
     char *home = getenv("HOME");
     if (home == NULL) {
@@ -622,6 +638,22 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
               inCharCode, inModifiers);
 #endif
 
+  if (CandidateWindowIsVisible(inSessionHandle) &&
+      inModifiers == 0 &&
+      (inCharCode >= '0' && inCharCode <= '9')) {
+    UInt32 row = inCharCode == '0' ? 9 : inCharCode - '1';
+#if DEBUG_KEYEVENT
+    DEBUG_PRINT("MUIMHandleInput() select candidate %lu index=%ld begin=%ld end=%ld\n",
+                row, (*inSessionHandle)->fCandidateIndex,
+                (*inSessionHandle)->fLayoutBegin,
+                (*inSessionHandle)->fNRCandidates);
+#endif
+    if ((*inSessionHandle)->fLayoutBegin + row <
+        (*inSessionHandle)->fNRCandidates)
+      CandClicked(row);
+    return TRUE;
+  }
+  
   /* Check for special keys first */
   for (i = 0; KeycodeToUKey[i].ukey; i++) {
     if (KeycodeToUKey[i].keycode == inKeycode) {
@@ -760,7 +792,7 @@ CreateUIMContext(MUIMSessionHandle inSessionHandle, CFStringRef inIMName)
   }
   else {
     // Load the preferences
-    imVal = CFPreferencesCopyAppValue(CFSTR("InputMethod"), CFSTR(kAppID));
+    imVal = CFPreferencesCopyAppValue(CFSTR(kPrefIM), CFSTR(kAppID));
     if (imVal && CFGetTypeID(imVal) == CFStringGetTypeID())
       imName = CFStringGetCStringPtr((CFStringRef) imVal,
                                      kCFStringEncodingMacRoman);
@@ -837,12 +869,25 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
                      CFDictionaryRef inUserInfo)
 {
   CFStringRef im;
-  UInt32 i;
+  CFBooleanRef on;
 
-  im = CFDictionaryGetValue(inUserInfo, CFSTR("InputMethod"));
+  im = CFDictionaryGetValue(inUserInfo, CFSTR(kPrefIM));
 
 #if DEBUG_NOTIFY
-  DEGUG_PRINT("NotificationCallback() im='%s'\n", CFStringGetCStringPtr(im, kCFStringEncodingMacRoman));
+  DEGUG_PRINT("NotificationCallback() im='%s'\n",
+              CFStringGetCStringPtr(im, kCFStringEncodingMacRoman));
+#endif
+  
+  on = CFDictionaryGetValue(inUserInfo, CFSTR(kPrefCandVertical));
+  gCandVertical = on == kCFBooleanTrue ? true : false;
+  
+  on = CFDictionaryGetValue(inUserInfo, CFSTR(kPrefModeTips));
+  gEnableModeTips = on == kCFBooleanTrue ? true : false;
+  
+#if DEBUG_NOTIFY
+  DEBUG_PRINT("NotificationCallback() vertical=%s modetips=%s\n",
+              gCandVertical ? "true" : "false",
+              gEnableModeTips ? "true" : "false");
 #endif
 
   if (im) {
@@ -854,11 +899,20 @@ NotificationCallback(CFNotificationCenterRef inCenter, void *inObserver,
     ReleaseAllUIMContext();
     CreateAllUIMContext(im);
 
-    gMode = -1;
-    for (i = 0; i < gNumSession; i++) {
-      uim_prop_list_update((*gSessionList[i])->fUC);
-      uim_prop_label_update((*gSessionList[i])->fUC);
+#ifdef SYNC_MODE
+    {
+      UInt32 i;
+
+      gMode = -1;
+      for (i = 0; i < gNumSession; i++) {
+        uim_prop_list_update((*gSessionList[i])->fUC);
+        uim_prop_label_update((*gSessionList[i])->fUC);
+      }
     }
+#else
+    uim_prop_list_update((*gActiveSession)->fUC);
+    uim_prop_label_update((*gActiveSession)->fUC);
+#endif
   }
 }
 
