@@ -62,16 +62,12 @@ convertHelperString(char *str);
 
 - initWithBundle:(NSBundle *)bundle
 {
-  NSNotificationCenter *noc;
-
   self = [super initWithBundle:bundle];
   if (!self)
     return nil;
 
   sharedExtra = self;
 
-  noc = [NSNotificationCenter defaultCenter];
-
   menu = [[NSMenu alloc] initWithTitle:@""];
   //[menu setAutoenablesItems:NO];
 
@@ -79,34 +75,9 @@ convertHelperString(char *str);
                                    menuExtra:self];
   [self setView:view];
 
-  uim_helper_client_get_prop_list();
+  //uim_helper_client_get_prop_list();
 
-#ifdef NEW_HELPER
-  if (uim_helper_init_client(uimDisconnect) == 0)
-#else
-  uimFD = uim_helper_init_client_fd(uimDisconnect);
-  if (uimFD >= 0)
-#endif
-  {
-    uimHandle = [[NSFileHandle alloc]
-#ifdef NEW_HELPER
-                  initWithFileDescriptor:uim_helper_get_client_fd()
-#else
-                  initWithFileDescriptor:uimFD
-#endif
-                 ];
-
-    [uimHandle waitForDataInBackgroundAndNotify];
-
-    [noc addObserver:self
-         selector:@selector(helperRead:)
-         name:@"NSFileHandleDataAvailableNotification"
-         object:uimHandle];
-
-#ifdef NEW_HELPER
-    is_helper_connect = YES;
-#endif
-  }
+  [self helperConnect];
 
   imName = nil;
 
@@ -146,9 +117,9 @@ convertHelperString(char *str);
   [view release];
   [menu release];
 
-  [self helperClose];
+  [self helperDisconnect];
 
-  [uimHandle release];
+  //[uimHandle release];
 
   [modes release];
   [propNames release];
@@ -397,20 +368,20 @@ convertHelperString(char *str);
       CFArrayRef array;
 
       im = [imNames objectAtIndex:i];
-      CFPreferencesSetAppValue(CFSTR("InputMethod"), im, CFSTR(kAppID));
+      CFPreferencesSetAppValue(CFSTR(kPrefIM), im, CFSTR(kAppID));
 
       array = CFArrayCreateMutable(kCFAllocatorDefault, 100, NULL);
       for (j = 0; j < [imNames count]; j++)
         CFArrayAppendValue(array, [imNames objectAtIndex:j]);
-      CFPreferencesSetAppValue(CFSTR("HelperIMSwitch"), array, CFSTR(kAppID));
+      CFPreferencesSetAppValue(CFSTR(kPrefHelperIM), array, CFSTR(kAppID));
 
       CFPreferencesAppSynchronize(CFSTR(kAppID));
 
       dict = CFDictionaryCreateMutable(kCFAllocatorDefault, 10,
                                        NULL, NULL);
 
-      CFDictionarySetValue(dict, CFSTR("InputMethod"), im);
-      CFDictionarySetValue(dict, CFSTR("HelperIMSwitch"), array);
+      CFDictionarySetValue(dict, CFSTR(kPrefIM), im);
+      CFDictionarySetValue(dict, CFSTR(kPrefHelperIM), array);
 
       center = CFNotificationCenterGetDistributedCenter();
       CFNotificationCenterPostNotification(center,
@@ -428,6 +399,40 @@ convertHelperString(char *str);
           arguments:[NSArray arrayWithObject:@"/Library/PreferencePanes/MacUIM.prefPane"]];
 }
 
+- (int)helperConnect
+{
+  NSNotificationCenter *noc = [NSNotificationCenter defaultCenter];
+
+#ifdef NEW_HELPER
+  if (uim_helper_init_client(uimDisconnect) == 0)
+#else
+  uimFD = uim_helper_init_client_fd(uimDisconnect);
+  if (uimFD >= 0)
+#endif
+  {
+    uimHandle = [[NSFileHandle alloc]
+#ifdef NEW_HELPER
+                  initWithFileDescriptor:uim_helper_get_client_fd()
+#else
+                  initWithFileDescriptor:uimFD
+#endif
+                 ];
+
+    [uimHandle waitForDataInBackgroundAndNotify];
+
+    [noc addObserver:self
+         selector:@selector(helperRead:)
+         name:@"NSFileHandleDataAvailableNotification"
+         object:uimHandle];
+
+#ifdef NEW_HELPER
+    is_helper_connect = YES;
+#endif
+  }
+
+  return uimFD;
+}
+
 - (void)helperRead:(NSNotification *)notification
 {
   char *tmp;
@@ -451,6 +456,14 @@ convertHelperString(char *str);
   if (uimFD >= 0)
 #endif
     [uimHandle waitForDataInBackgroundAndNotify];
+  else {
+    // disconnected
+#if DEBUG_HELPER_EXTRA
+    fprintf(stderr, "MacUIMHelperExtra::helperRead() disconnected\n");
+#endif
+    [self helperDisconnect];
+    [self helperConnect];
+  }
   
   //[self loadPrefs];
 }
@@ -524,6 +537,9 @@ convertHelperString(char *str);
 {
   [[NSNotificationCenter defaultCenter] removeObserver:self];
 
+  [uimHandle release];
+  uimHandle = nil;
+
 #ifdef NEW_HELPER
   is_helper_connect = NO;
 #else
@@ -626,7 +642,7 @@ convertHelperString(char *str);
   
   CFPreferencesAppSynchronize(CFSTR(kAppID));
 
-  propVal = CFPreferencesCopyAppValue(CFSTR("InputMethod"), CFSTR(kAppID));
+  propVal = CFPreferencesCopyAppValue(CFSTR(kPrefIM), CFSTR(kAppID));
   if (!propVal || CFGetTypeID(propVal) != CFStringGetTypeID())
     return;
 
@@ -634,7 +650,7 @@ convertHelperString(char *str);
 
   imName = (NSString *) propVal;
 
-  propVal = CFPreferencesCopyAppValue(CFSTR("HelperIMSwitch"), CFSTR(kAppID));
+  propVal = CFPreferencesCopyAppValue(CFSTR(kPrefHelperIM), CFSTR(kAppID));
   if (!propVal || CFGetTypeID(propVal) != CFArrayGetTypeID())
     return;
   
