@@ -33,6 +33,7 @@
 
 #include <uim/uim.h>
 #include <uim/uim-helper.h>
+#include <uim/uim-im-switcher.h>
 
 #include "MUIM.h"
 #include "MUIMInputEvents.h"
@@ -56,6 +57,9 @@ extern CFSocketRef gUimSock;
 extern CFRunLoopSourceRef gUimRun;
 extern CFSocketContext gSockContext;
 
+extern Boolean gEnableModeTips;
+extern Boolean gCandVertical;
+
 static void
 AddPreeditSegment(MUIMSessionHandle inHandle, int inAttr,
                   const char *inStr);
@@ -67,6 +71,8 @@ HelperRead(CFSocketRef sock, CFSocketCallBackType callbackType,
 static void
 ParseHelperString(const char *str);
 
+static void
+ParseIMChangeString(const char *str);
 
 void
 UIMCommitString(void *ptr, const char *str)
@@ -349,7 +355,7 @@ UIMUpdatePropLabel(void *inPtr, const char *inStr)
     free(tmp);
   }
 
-  {
+  if (gEnableModeTips) {
     CFStringRef allstr;
     CFArrayRef array;
 
@@ -481,53 +487,79 @@ ParseHelperString(const char *str)
   DEBUG_PRINT("ParseHelperString() str='%s'\n", str);
 #endif
 
-  if (gActiveSession) {
-    if (strncmp("prop_list_get", str, 13) == 0) {
-      //DEBUG_PRINT("ParseHelperString() prop_list_get\n");
-      for (i = 0; i < gNumSession; i++)
-        uim_prop_list_update((*gActiveSession)->fUC);
-    }
-    else if (strncmp("prop_label_get", str, 14) == 0) {
-      //DEBUG_PRINT("ParseHelperString() prop_label_get\n");
-      for (i = 0; i < gNumSession; i++)
-        uim_prop_label_update((*gActiveSession)->fUC);
-    }
-    else if (strncmp("prop_activate", str, 13) == 0) {
-      CFMutableStringRef cfstr;
-      CFArrayRef array;
-      CFStringRef first;
+  if (!gActiveSession) return;
 
-      //DEBUG_PRINT("ParseHelperString() prop_activate\n");
 
-      cfstr = CFStringCreateMutable(NULL, 0);
-      CFStringAppendCString(cfstr, str, kCFStringEncodingUTF8);
+  if (strncmp("im_change", str, 9) == 0) {
+    ParseIMChangeString(str);
+  }
+  else if (strncmp("prop_list_get", str, 13) == 0) {
+    //for (i = 0; i < gNumSession; i++)
+    uim_prop_list_update((*gActiveSession)->fUC);
+  }
+  else if (strncmp("prop_label_get", str, 14) == 0) {
+    //for (i = 0; i < gNumSession; i++)
+    uim_prop_label_update((*gActiveSession)->fUC);
+  }
+  else if (strncmp("prop_activate", str, 13) == 0) {
+    CFMutableStringRef cfstr;
+    CFArrayRef array;
+    CFStringRef first;
 
-      array = CFStringCreateArrayBySeparatingStrings(kCFAllocatorDefault,
-                                                     cfstr, CFSTR("\n"));
+    cfstr = CFStringCreateMutable(NULL, 0);
+    CFStringAppendCString(cfstr, str, kCFStringEncodingUTF8);
 
-      if (array && CFArrayGetCount(array) > 0 &&
-          (first = CFArrayGetValueAtIndex(array, 0))) {
-        CFStringRef second = CFArrayGetValueAtIndex(array, 1);
-        if (second) {
+    array = CFStringCreateArrayBySeparatingStrings(kCFAllocatorDefault,
+                                                   cfstr, CFSTR("\n"));
+
+    if (array && CFArrayGetCount(array) > 0 &&
+        (first = CFArrayGetValueAtIndex(array, 0))) {
+      CFStringRef second = CFArrayGetValueAtIndex(array, 1);
+      if (second) {
 #if DEBUG_HELPER
-          DEBUG_PRINT("ParseHelperString() first='%s' second='%s'\n",
-                      CFStringGetCStringPtr(first, kCFStringEncodingUTF8),
-                      CFStringGetCStringPtr(second, kCFStringEncodingUTF8));
+        DEBUG_PRINT("ParseHelperString() first='%s' second='%s'\n",
+                    CFStringGetCStringPtr(first, kCFStringEncodingUTF8),
+                    CFStringGetCStringPtr(second, kCFStringEncodingUTF8));
 #endif
-          for (i = 0; i < gNumSession; i++)
-            uim_prop_activate((*gSessionList[i])->fUC,
-                              CFStringGetCStringPtr(second, kCFStringEncodingUTF8));
-          CFRelease(second);
-        }
-        CFRelease(first);
+        for (i = 0; i < gNumSession; i++)
+          uim_prop_activate((*gSessionList[i])->fUC,
+                            CFStringGetCStringPtr(second, kCFStringEncodingUTF8));
+        CFRelease(second);
       }
+      CFRelease(first);
     }
-    else if (strncmp("prop_list_update", str, 16) == 0) {
-    }
-    else if (strncmp("prop_label_update", str, 17) == 0) {
-    }
-    else if (strncmp("focus_in", str, 8) == 0) {
-    }
+  }
+  else if (strncmp("prop_list_update", str, 16) == 0) {
+  }
+  else if (strncmp("prop_label_update", str, 17) == 0) {
+  }
+  else if (strncmp("focus_in", str, 8) == 0) {
+  }
+}
+
+static void
+ParseIMChangeString(const char *str)
+{
+  CFStringRef allstr;
+  CFArrayRef array;
+  CFStringRef imName;
+  UInt32 i;
+  
+  allstr = CFStringCreateWithCString(kCFAllocatorDefault,
+                                     str, kCFStringEncodingUTF8);
+  array = CFStringCreateArrayBySeparatingStrings(kCFAllocatorDefault,
+                                                 allstr, CFSTR("\n"));
+
+  imName = CFArrayGetValueAtIndex(array, 1);
+
+#if DEBUG_HELPER
+  DEBUG_PRINT("ParseIMChangeString() imName='%s'\n",
+              CFStringGetCStringPtr(imName, kCFStringEncodingUTF8));
+#endif
+  
+  for (i = 0; i < gNumSession; i++) {
+    uim_switch_im((*gSessionList[i])->fUC,
+                   CFStringGetCStringPtr(imName, kCFStringEncodingUTF8));
   }
 }
 
